/*------------------------------------------------------------------------------
--  This file is a part of the VHDL Tools Software
--  Copyright (C) 2014, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#include "vhdl_scanner.h"
#include <QDebug>

VHDL_Tools::vhdl_Scanner::vhdl_Scanner(std::istream *in, const QString &fileName)
    : yyFlexLexer(in)
{
    this->rootNode = new VHDL_Tools::VHDL_AST_Node(fileName,VHDL_Tools::rootNode);
    this->currentNode = rootNode;
    this->rootNode->parent = this->rootNode;
}

VHDL_Tools::vhdl_Scanner::~vhdl_Scanner()
{
    deleteNode(rootNode);
}


int VHDL_Tools::vhdl_Scanner::scan()
{
    return( yylex() );
}

int VHDL_Tools::vhdl_Scanner::newFile(std::istream *in, const QString &fileName,bool trashPreviousTree)
{
    switch_streams(in);
    if(trashPreviousTree)
        deleteNode(rootNode);
    this->rootNode = new VHDL_Tools::VHDL_AST_Node(fileName,VHDL_Tools::rootNode);
    this->currentNode = rootNode;
    this->rootNode->parent = this->rootNode;
    return 1;
}


void VHDL_Tools::vhdl_Scanner::appendNode(VHDL_Tools::VHDL_AST_Node *node)
{
    this->currentNode->childs.append(node);
    node->parent = this->currentNode;
    this->currentNode = node;
}

void VHDL_Tools::vhdl_Scanner::deleteNode(VHDL_Tools::VHDL_AST_Node *node)
{
    for(int i=0;i<node->childs.count();i++)
    {
        deleteNode(node->childs.at(i));
    }
    node->parent->childs.removeAll(node);
    delete node;
}
